<?php

namespace Drupal\usage_data\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\usage_data\Plugin\UsageTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Usage Data settings form management.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * Usage type manager.
   *
   * @var \Drupal\usage_data\Plugin\UsageTypeManagerInterface
   */
  protected UsageTypeManagerInterface $usageTypeManager;

  /**
   * Plugin definitions.
   *
   * @var array
   */
  protected array $definitions;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Creates a SettingsForm instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory class instance.
   * @param \Drupal\usage_data\Plugin\UsageTypeManagerInterface $usageTypeManager
   *   The usage type manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(ConfigFactoryInterface $config_factory, UsageTypeManagerInterface $usageTypeManager, ModuleHandlerInterface $module_handler) {
    parent::__construct($config_factory);
    $this->usageTypeManager = $usageTypeManager;
    $this->moduleHandler = $module_handler;
    $this->definitions = $this->usageTypeManager->getDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.usage_type'),
      $container->get('module_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'usage_data_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);

    foreach ($this->definitions as $definition) {
      $id = $definition['id'];
      $config = $this->config('usage_data.settings')->get($id);
      $form[$id] = [
        '#type' => 'details',
        '#title' => $definition['label'],
        '#collapsible' => TRUE,
        '#open' => TRUE,
      ];
      $form[$id][$id . '_enabled'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable plugin'),
        '#default_value' => isset($config['enabled']) ? $config['enabled'] : FALSE,
      ];
      $form[$id][$id . '_redis_queue'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Use Redis queue'),
        '#description' => $this->t('1. Enable and setup <a href="https://www.drupal.org/project/redis" target="_blank">Redis</a> module.<br>
          2. Add to your settings.php:<br>$settings[\'queue_service_usage_data_records\'] = \'queue.redis\';'),
        '#default_value' => isset($config['redis_queue']) ? $config['redis_queue'] : FALSE,
        '#disabled' => !$this->moduleHandler->moduleExists('redis'),
      ];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('usage_data.settings');

    foreach ($this->definitions as $definition) {
      $id = $definition['id'];
      $values = $config->get($id);
      $values['enabled'] = $form_state->getValue($id . '_enabled');
      $values['redis_queue'] = $form_state->getValue($id . '_redis_queue');
      $config->set($id, $values);
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['usage_data.settings'];
  }

}
